<?php
// +--------------------------------------------------------------------------+
// | Image_Graph                                                              |
// +--------------------------------------------------------------------------+
// | Copyright (C) 2003, 2004 Jesper Veggerby                                 |
// | Email         pear.nosey@veggerby.dk                                     |
// | Web           http://pear.veggerby.dk                                    |
// | PEAR          http://pear.php.net/package/Image_Graph                    |
// +--------------------------------------------------------------------------+
// | This library is free software; you can redistribute it and/or            |
// | modify it under the terms of the GNU Lesser General Public               |
// | License as published by the Free Software Foundation; either             |
// | version 2.1 of the License, or (at your option) any later version.       |
// |                                                                          |
// | This library is distributed in the hope that it will be useful,          |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        |
// | Lesser General Public License for more details.                          |
// |                                                                          |
// | You should have received a copy of the GNU Lesser General Public         |
// | License along with this library; if not, write to the Free Software      |
// | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA |
// +--------------------------------------------------------------------------+

/**
 * Image_Graph - PEAR PHP OO Graph Rendering Utility.
 *
 * @package Image_Graph
 * @subpackage Text
 * @category images
 * @copyright Copyright (C) 2003, 2004 Jesper Veggerby Hansen
 * @license http://www.gnu.org/licenses/lgpl.txt GNU Lesser General Public License
 * @author Jesper Veggerby <pear.nosey@veggerby.dk>
 * @version $Id: TTF.php,v 1.4 2004/12/19 22:17:37 neufeind Exp $
 */

/**
 * Include file Image/Graph/Font/Extended.php
 */
require_once 'Image/Graph/Font/Extended.php';

/**
 * A truetype font.
 *
 * @author Jesper Veggerby <pear.nosey@veggerby.dk>
 * @package Image_Graph
 * @subpackage Text
 */
class Image_Graph_Font_TTF extends Image_Graph_Font_Extended
{

    /**
     * The file of the font.
     * On Windows systems they will be located in %SYSTEMROOT%\FONTS, ie C:\WINDOWS\FONTS
     * @var string
     * @access private
     */
    var $_fontFile;

    /**
     * FontTTF [Constructor]
     *
     * @param string $fontFile The filename of the TTF font file. On Windows
     *   systems they will be located in %SYSTEMROOT%\FONTS, ie C:\WINDOWS\FONTS
     */
    function &Image_Graph_Font_TTF($fontFile)
    {
        parent::Image_Graph_Font();
        $this->setFontFile($fontFile);
    }

    /**
     * Set another font file
     *
     * @param string $fontFile The filename of the TTF font file. On Windows
     *   systems they will be located in %SYSTEMROOT%\FONTS, ie C:\WINDOWS\FONTS
     */
    function setFontFile($fontFile)
    {
        $this->_fontFile = $fontFile;
    }

    /**
     * Get the font 'array'
     *
     * @return array The font 'summary' to pass to the driver
     * @access private
     */
    function _getFont($options = false)
    {
        $options = parent::_getFont($options);
        unset($options['font']);
        $options['ttf'] = $this->_fontFile;
        if (!isset($options['size'])) {
            $options['size'] = $this->_size;
        }
        if (!isset($options['angle'])) {
            $options['angle'] = $this->_angle;
        }
        return $options;
    }

}

?>